@@ -62,7 +62,7 @@ Show titled images side-by-side:
 Display a video (an iterable of images, e.g., a 3D or 4D array):
 ```python
   video = moving_circle((100, 100), num_images=10)
-  show_video(video)
+  show_video(video, fps=10)
 ```
 
 Show the video frames side-by-side:
@@ -78,7 +78,7 @@ Show the frames with their indices:
 Read and display a video (either local or from the Web):
 ```python
   VIDEO = 'https://github.com/hhoppe/data/raw/main/video.mp4'
-  show_video(read_video(VIDEO))
+  show_video(read_video(VIDEO), fps=60)
 ```
 
 Create and display a looping two-frame GIF video:
@@ -95,11 +95,11 @@ Darken a video frame-by-frame:
     with VideoWriter(output_path, shape=r.shape, fps=r.fps, bps=r.bps) as w:
       for image in r:
         w.add_image(darken_image(image))
-  show_video(read_video(output_path), height=90)
 ```
 """
 
-__version__ = '0.1.3'
+__docformat__ = 'google'
+__version__ = '0.2.0'
 __version_info__ = tuple(int(num) for num in __version__.split('.'))
 
 import base64
@@ -141,11 +141,17 @@ if typing.TYPE_CHECKING:
 else:
   _Path = Union[str, os.PathLike]
 
-_ffmpeg_name_or_path: _Path = 'ffmpeg'
-
 ## Miscellaneous.
 
 
+class _Config:
+  ffmpeg_name_or_path: _Path = 'ffmpeg'
+  show_save_dir: Optional[_Path] = None
+
+
+_config = _Config()
+
+
 def _open(path: _Path, *args: Any, **kwargs: Any) -> ContextManager[Any]:
   """Opens the file; this is a hook for the built-in open()."""
   return open(path, *args, **kwargs)
@@ -159,7 +165,7 @@ def _path_is_local(path: _Path) -> bool:
 
 def _search_for_ffmpeg_path() -> Optional[str]:
   """Returns a path to the ffmpeg program, or None if not found."""
-  filename = shutil.which(_ffmpeg_name_or_path)
+  filename = shutil.which(_config.ffmpeg_name_or_path)
   if filename:
     return filename
   return None
@@ -243,8 +249,7 @@ def set_ffmpeg(name_or_path: _Path) -> None:
     name_or_path: Either a filename within a directory of `os.environ['PATH']`
       or a filepath.  The default setting is 'ffmpeg'.
   """
-  global _ffmpeg_name_or_path
-  _ffmpeg_name_or_path = name_or_path
+  _config.ffmpeg_name_or_path = name_or_path
 
 
 def set_output_height(num_pixels: int) -> None:
@@ -628,38 +633,35 @@ def _write_via_local_file(path: _Path) -> Generator[str, None, None]:
         f.write(tmp_path.read_bytes())
 
 
-class _ShowSave:
-  """Saves outputs from `show_image` and `show_video` into files."""
-  dir: Optional[_Path] = None
+class set_show_save_dir:  # pylint: disable=invalid-name
+  """Save all titled output from `show_*()` calls into files.
 
-  @contextlib.contextmanager
-  def to_dir(self, path: _Path) -> Generator[None, None, None]:
-    """Temporarily sets output directory for show_image() and show_video()."""
-    previous_dir = self.dir
-    try:
-      self.dir = path
-      yield
-    finally:
-      self.dir = previous_dir
+  If the specified `directory` is not None, all titled images and videos
+  displayed by `show_image`, `show_images`, `show_video`, and `show_videos` are
+  also saved as files within the directory.
 
+  It can be used either to set the state or as a context manager:
 
-show_save = _ShowSave()
-"""Functionality to save all titled output from `show_*()` calls into files.
+  >>> set_show_save_dir('/tmp')
+  >>> show_image(image, title='image1')  # Creates /tmp/image1.png.
+  >>> show_video(video, title='video2')  # Creates /tmp/video2.mp4.
+  >>> set_show_save_dir(None)
 
-If the `dir` attribute of `show_save` is not None, all titled images and videos
-displayed by `show_image`, `show_images`, `show_video`, and `show_videos` are
-also saved as files within the specified directory.
+  >>> with set_show_save_dir('/tmp'):
+  ...   show_image(image, title='image1')  # Creates /tmp/image1.png.
+  ...   show_video(video, title='video2')  # Creates /tmp/video2.mp4.
+  """
 
-The context `to_dir(directory_name)` temporarily assigns the
-`dir` attribute:
+  def __init__(self, directory: Optional[_Path]):
+    self._old_show_save_dir = _config.show_save_dir
+    _config.show_save_dir = directory
 
->>> with show_save.to_dir('/tmp'):
-...   show_image(image, title='image1')  # Creates /tmp/image1.png.
-...   show_video(video, title='video2')  # Creates /tmp/video2.mp4.
+  def __enter__(self) -> None:
+    pass
+
+  def __exit__(self, *_: Any) -> None:
+    _config.show_save_dir = self._old_show_save_dir
 
-Attributes:
-  dir: Directory into which to save titled images and videos, or None.
-"""
 
 ## Image I/O.
 
@@ -684,6 +686,7 @@ def write_image(path: _Path, image: np.ndarray, **kwargs: Any) -> None:
       Otherwise it must be np.uint8 or np.uint16.
     **kwargs: Additional parameters for `PIL.Image.save()`.
   """
+  image = _as_valid_media_array(image)
   if issubclass(image.dtype.type, np.floating):
     image = to_uint8(image)
   with _open(path, 'wb') as f:
@@ -744,6 +747,7 @@ def compress_image(image: np.ndarray,
     **kwargs: Options for `PIL.save()`, e.g. `optimize=True` for greater
       compression.
   """
+  image = _as_valid_media_array(image)
   with io.BytesIO() as output:
     _pil_image(image).save(output, format=fmt, **kwargs)
     return output.getvalue()
@@ -837,8 +841,8 @@ def show_images(
 ) -> None:
   """Displays a row of images in the IPython/Jupyter notebook.
 
-  If `show_save`.`dir` is not None, saves each titled image to a file based
-  on the title.
+  If a directory has been specified using `set_show_save_dir`, also saves each
+  titled image to a file in that directory based on its title.
 
   >>> image1, image2 = np.random.rand(64, 64, 3), color_ramp((64, 64))
   >>> show_images([image1, image2])
@@ -867,14 +871,11 @@ def show_images(
       raise ValueError('Cannot have images dictionary and titles parameter.')
     list_titles, list_images = list(images.keys()), list(images.values())
   else:
-    list_images: List[np.ndarray] = list(images)  # type: ignore
-    if titles is None:
-      list_titles = [None] * len(list_images)
-    else:
-      list_titles = list(titles)
-      if len(list_images) != len(list_titles):
-        raise ValueError('Number of images does not match number of titles'
-                         f' ({len(list_images)} vs {len(list_titles)}).')
+    list_images = list(images)
+    list_titles = [None] * len(list_images) if titles is None else list(titles)
+    if len(list_images) != len(list_titles):
+      raise ValueError('Number of images does not match number of titles'
+                       f' ({len(list_images)} vs {len(list_titles)}).')
 
   def ensure_mapped_to_rgb(image: Any) -> np.ndarray:
     image = _as_valid_media_array(image)
@@ -901,8 +902,8 @@ def show_images(
   png_datas = [compress_image(to_uint8(image)) for image in list_images]
 
   for title, png_data in zip(list_titles, png_datas):
-    if title and show_save.dir:
-      path = pathlib.Path(show_save.dir) / f'{title}.png'
+    if title and _config.show_save_dir:
+      path = pathlib.Path(_config.show_save_dir) / f'{title}.png'
       with _open(path, mode='wb') as f:
         f.write(png_data)
 
@@ -940,7 +941,7 @@ def _get_ffmpeg_path() -> str:
   path = _search_for_ffmpeg_path()
   if not path:
     raise RuntimeError(
-        f"Program '{_ffmpeg_name_or_path}' is not found;"
+        f"Program '{_config.ffmpeg_name_or_path}' is not found;"
         " perhaps install ffmpeg using 'apt-get install ffmpeg'.")
   return path
 
@@ -1042,6 +1043,9 @@ class VideoReader(_VideoIO, ContextManager[Any]):
   ...   for image in reader:
   ...     print(image.shape)
 
+  >>> with VideoReader('/tmp/river.mp4') as reader:
+  ...   video = np.array(tuple(reader))
+
   Attributes:
     path_or_url: Location of input video.
     output_format: Format of output images (default 'rgb'):
@@ -1162,7 +1166,7 @@ class VideoWriter(_VideoIO, ContextManager[Any]):
   >>> with VideoWriter('/tmp/v.mp4', shape, fps=60) as writer:
   ...   for image in moving_circle(shape, num_images=60):
   ...     writer.add_image(image)
-  >>> show_video(read_video('/tmp/v.mp4'))
+  >>> show_video(read_video('/tmp/v.mp4'), fps=60)
 
 
   Bitrate control may be specified using at most one of: `bps`, `qp`, or `crf`.
@@ -1178,7 +1182,7 @@ class VideoWriter(_VideoIO, ContextManager[Any]):
     shape: 2D spatial dimensions (height, width) of video image frames.  The
       dimensions must be even if 'encoded_format' has subsampled chroma (e.g.,
       'yuv420p' or 'yuv420p10le').
-    fps: Frames-per-second frame rate (default is 60.0 except 25.0 for 'gif').
+    fps: Frames-per-second framerate (default is 60.0 except 25.0 for 'gif').
     codec: Compression algorithm as defined by "ffmpeg -codecs" (e.g., 'h264',
       'hevc', 'vp9', or 'gif').
     bps: Requested average bits-per-second bitrate (default None).
@@ -1363,8 +1367,14 @@ def read_video(path_or_url: _Path, **kwargs: Any) -> np.ndarray:
 
   >>> video = read_video('/tmp/river.mp4')
   >>> show_video(video)
+
   >>> url = 'https://github.com/hhoppe/data/raw/main/video.mp4'
-  >>> show_video(read_video(url))
+  >>> show_video(read_video(url), fps=60)
+
+  To obtain the original framerate, one may use:
+
+  >>> with VideoReader(url) as reader:
+  ...   show_video(reader, fps=reader.fps)
 
   Args:
     path_or_url: Input video file.
@@ -1382,8 +1392,8 @@ def write_video(path: _Path, images: Iterable[np.ndarray],
   """Writes images to a compressed video file.
 
   >>> video = moving_circle((480, 640), num_images=60)
-  >>> write_video('/tmp/v.mp4', video, fps=60.0, qp=18)
-  >>> show_video(read_video('/tmp/v.mp4'))
+  >>> write_video('/tmp/v.mp4', video, fps=60, qp=18)
+  >>> show_video(read_video('/tmp/v.mp4'), fps=60)
 
   Args:
     path: Output video file.
@@ -1485,13 +1495,15 @@ def show_video(images: Iterable[np.ndarray],
   See `show_videos`.
 
   >>> video = read_video('https://github.com/hhoppe/data/raw/main/video.mp4')
-  >>> show_video(video, title='River video', fps=10.0)
+  >>> show_video(video, title='River video', fps=10)
 
-  >>> show_video(moving_circle((80, 80), num_images=10), border=True)
-  >>> show_video(read_video('/tmp/river.mp4'))
+  >>> show_video(moving_circle((80, 80), num_images=10), fps=5, border=True)
+
+  >>> show_video(read_video('/tmp/river.mp4'), fps=60)
 
   Args:
-    images: Iterable of video frames.
+    images: Iterable of video frames (e.g., a 4D array or a list of 2D or 3D
+      arrays).
     title: Optional text shown centered above the video.
     **kwargs: See `show_videos`.
   """
@@ -1500,7 +1512,7 @@ def show_video(images: Iterable[np.ndarray],
 
 def show_videos(videos: Union[Iterable[Iterable[np.ndarray]],
                               Mapping[str, Iterable[np.ndarray]]],
-                titles: Optional[Sequence[Optional[str]]] = None,
+                titles: Optional[Iterable[Optional[str]]] = None,
                 *,
                 width: Optional[int] = None,
                 height: Optional[int] = None,
@@ -1519,20 +1531,20 @@ def show_videos(videos: Union[Iterable[Iterable[np.ndarray]],
   when the `fps` period is not a multiple of 10 ms units (GIF frame delay
   units).  Encoding at `fps` = 20.0, 25.0, or 50.0 works fine.
 
-  If `show_save`.`dir` is not None, saves each titled video to a file based
-  on the title.
+  If a directory has been specified using `set_show_save_dir`, also saves each
+  titled video to a file in that directory based on its title.
 
   Args:
     videos: Iterable of videos, or dictionary of `{title: video}`.  Each video
       must be an iterable of images.
-    titles: Optional sequence of strings shown above the corresponding videos.
+    titles: Optional strings shown above the corresponding videos.
     width: Optional, overrides displayed width (in pixels).
     height: Optional, overrides displayed height (in pixels).
     downsample: If True, each video whose width or height is greater than the
       specified `width` or `height` is resampled to the display resolution. This
       improves antialiasing and reduces the size of the notebook.
     columns: Optional, maximum number of videos per row.
-    fps: Frames-per-second frame rate (default is 60.0 except 25.0 for GIF).
+    fps: Frames-per-second framerate (default is 60.0 except 25.0 for GIF).
     bps: Bits-per-second bitrate (default None).
     qp: Quantization parameter for video compression quality (default None).
     codec: Compression algorithm; must be either 'h264' or 'gif'.
@@ -1545,7 +1557,7 @@ def show_videos(videos: Union[Iterable[Iterable[np.ndarray]],
           'Cannot have both a video dictionary and a titles parameter.')
     list_titles, list_videos = list(videos.keys()), list(videos.values())
   else:
-    list_videos: List[Iterable[np.ndarray]] = list(videos)  # type: ignore
+    list_videos = typing.cast(List[Iterable[np.ndarray]], list(videos))
     list_titles = [None] * len(list_videos) if titles is None else list(titles)
     if len(list_videos) != len(list_titles):
       raise ValueError('Number of videos does not match number of titles'
@@ -1561,9 +1573,9 @@ def show_videos(videos: Union[Iterable[Iterable[np.ndarray]],
       # Not resize_video() because each image may have different depth and type.
       video = [resize_image(image, (h, w)) for image in video]
     data = compress_video(video, fps=fps, bps=bps, qp=qp, codec=codec)
-    if title and show_save.dir:
+    if title and _config.show_save_dir:
       suffix = _filename_suffix_from_codec(codec)
-      path = pathlib.Path(show_save.dir) / f'{title}{suffix}'
+      path = pathlib.Path(_config.show_save_dir) / f'{title}{suffix}'
       with _open(path, mode='wb') as f:
         f.write(data)
     if codec == 'gif':
